from flask import url_for
from urlparse import urljoin

def get_app_url(config):
  """ Returns the application's URL, based on the given config. """
  return '%s://%s' % (config['PREFERRED_URL_SCHEME'], config['SERVER_HOSTNAME'])

def get_app_url_from_scheme_hostname(url_scheme_and_hostname):
  """ Returns the application's URL, based on the given url scheme and hostname. """
  return '%s://%s' % (url_scheme_and_hostname.url_scheme, url_scheme_and_hostname.hostname)

def slash_join(*args):
  """
  Joins together strings and guarantees there is only one '/' in between the
  each string joined. Double slashes ('//') are assumed to be intentional and
  are not deduplicated.
  """
  def rmslash(path):
    path = path[1:] if len(path) > 0 and path[0] == '/' else path
    path = path[:-1] if len(path) > 0 and path[-1] == '/' else path
    return path

  args = [rmslash(path) for path in args]
  return '/'.join(args)

def create_uri_func_from_context(context, url_scheme_and_hostname):
  def create_uri(repository_and_namespace, checksum):
    with context:
      relative_layer_url = url_for('v2.download_blob', repository=repository_and_namespace,
                                   digest=checksum)
    return urljoin(get_app_url_from_scheme_hostname(url_scheme_and_hostname), relative_layer_url)

  return create_uri

